#pragma once

#include <iostream>
#include <string>

#include <boost/mpl/vector.hpp>
#include <boost/msm/back/state_machine.hpp>
#include <boost/msm/front/state_machine_def.hpp>



namespace test
{

    namespace event
    {
        struct exit{};

    }

    
    class StateBase
        : public boost::msm::front::state<>
    {
    public:

        StateBase( const std::string& state_name  )
            : m_state_name( state_name )
        {
            std::cout << "--> Create " << name() << " .\n";
        }

        ~StateBase()
        {
            std::cout << "--> Destroy " << name() << " .\n";
        }

        template< class Event, class FSM >
        void on_entry( const Event&, FSM&  )
        {
            std::cout << "==> Enter " << name() << " .\n";
        }

        template< class Event, class FSM >
        void on_exit( const Event&, FSM&  )
        {
            std::cout << "==> Exit " << name() << " .\n";
        }

        const std::string& name() const { return m_state_name; }

    private:

        std::string m_state_name;
    };

    class State_HelloWorld 
        : public StateBase
    {
    public:
        State_HelloWorld() : StateBase( "HelloWorld" ){}
    };

    class State_Exit
        : public StateBase
    {
    public:

        State_Exit() : StateBase( "Exit" ){}

    };

    class StateMachine
        : public boost::msm::front::state_machine_def< StateMachine >
    {
    public:

        typedef State_HelloWorld initial_state;

        struct transition_table : public boost::mpl::vector
            <
                _row< State_HelloWorld, event::exit, State_Exit >
            >
        {};

        StateMachine();
        ~StateMachine();
    };

    class RealStateMachine
        : public boost::msm::back::state_machine< StateMachine >
    {
    public:

        


    };

}